package leben.ernaehrung;

import leben.lebewesen.Lebewesen;

/**
 * Implementierung der
 * Multi-Ern&auml;hrung.
 * 
 * @author Heiner K&uuml;cker
 */
public final class MultiErnaehrung
	extends AbstractErnaehrung
{
	private final AbstractErnaehrung[] ernaehrungArr;
	
	/**
	 * Konstruktor f&uuml;r mehrere Ern&auml;hrungs-M&ouml;glichkeiten.
	 * 
	 * @param ErnaehrungArr Array mit den Ern&auml;hrungs-M&ouml;glichkeiten
	 */
	public MultiErnaehrung(
			final AbstractErnaehrung... ernaehrungArr )
	{
		// TODO Verhindern null-Array und null-Elemente
		this.ernaehrungArr = ernaehrungArr;
	}



	/**
	 * Multi-Ern&auml;hrungs-Methode.
	 * 
	 * @param lebewesenToErnaehr zu ern&auml;hrendes Lebewesen
	 * @see AbstractErnaehrung#ernaehreDich(Lebewesen)
	 */
	@Override
	public void ernaehreDich(
			final Lebewesen lebewesenToErnaehr )
	{
		for ( AbstractErnaehrung ernaehrung : this.ernaehrungArr )
		{
			ernaehrung.ernaehreDich( lebewesenToErnaehr );
		}
	}
}
