package leben.lebewesen;

import java.util.ArrayList;
import java.util.List;

import leben.bewegung.AbstractBewegung;
import leben.bewegung.FliegBewegung;
import leben.bewegung.LaufBewegung;
import leben.bewegung.MultiBewegung;
import leben.bewegung.SchwimmBewegung;
import leben.ernaehrung.AbstractErnaehrung;
import leben.ernaehrung.MultiErnaehrung;
import leben.ernaehrung.PflanzlichErnaehrung;
import leben.ernaehrung.TierischErnaehrung;
import leben.fortpflanzung.AbstractFortpflanzung;
import leben.fortpflanzung.FischFortpflanzung;
import leben.fortpflanzung.SaeugetierFortpflanzung;
import leben.fortpflanzung.VogelFortpflanzung;

/**
 * Lebewesen-Klasse.
 * <br /><br />
 * Das Lebewesen kann sich auf
 * unterschiedliche Art
 * <br />
 * bewegen ({@link AbstractBewegung),
 * <br />
 * ern&auml;hren ({@link AbstractErnaehrung),
 * <br />
 * und fortpflanzen ({@link AbstractFortpflanzung).
 * 
 * @author Heiner K&uuml;cker
 */
public final class Lebewesen
{
	/**
	 * Name des Lebewesens.
	 * <br />
	 * Hier als public Member, weil final,
	 * aber falls man noch mit Vererbung dieser
	 * Klasse arbeiten will, dann besser mit
	 * get-Methode.
	 */
	public final String name;

	/**
	 * Bewegung des Lebewesens.
	 * <br />
	 * Hier als public Member, weil final,
	 * aber falls man noch mit Vererbung dieser
	 * Klasse arbeiten will, dann besser mit
	 * get-Methode.
	 */
	public final AbstractBewegung bewegung;
	
	/**
	 * Ern&auml;hrung des Lebewesens.
	 * <br />
	 * Hier als public Member, weil final,
	 * aber falls man noch mit Vererbung dieser
	 * Klasse arbeiten will, dann besser mit
	 * get-Methode.
	 */
	public final AbstractErnaehrung ernaehrung;
	
	/**
	 * Fortpflanzung des Lebewesens.
	 * <br />
	 * Hier als public Member, weil final,
	 * aber falls man noch mit Vererbung dieser
	 * Klasse arbeiten will, dann besser mit
	 * get-Methode.
	 */
	public final AbstractFortpflanzung fortpflanzung;

	/**
	 * Konstruktor.
	 * 
	 * @param name			Name des Lebewesens
	 * @param bewegung		Bewegung des Lebewesens
	 * @param ernaehrung	Ern&auml;hrung des Lebewesens
	 * @param fortpflanzung	Fortpflanzung des Lebewesens
	 */
	public Lebewesen(
			final String name , 
			final AbstractBewegung bewegung ,
			final AbstractErnaehrung ernaehrung , 
			final AbstractFortpflanzung fortpflanzung )
	{
		// TODO null-Parameter abweisen
		this.name = name;
		this.bewegung = bewegung;
		this.ernaehrung = ernaehrung;
		this.fortpflanzung = fortpflanzung;
	}
	
	/**
	 * Bewegungs-Methode.
	 * 
	 * @see AbstractBewegung#bewegDich(Lebewesen)
	 */
	public void bewegDich()
	{
		this.bewegung.bewegDich( this );
	}
	
	/**
	 * Ern&auml;hrungs-Methode.
	 * 
	 * @see AbstractErnaehrung#ernaehreDich(Lebewesen)
	 */
	public void ernaehreDich()
	{
		this.ernaehrung.ernaehreDich( this );
	}

	/**
	 * Fortpflanzungs-Methode.
	 * 
	 * @see leben.fortpflanzung.AbstractFortpflanzung#bewegDich(leben.lebewesen.Lebewesen)
	 */
	public void fortpflanzDich()
	{
		this.fortpflanzung.fortpflanzDich( this );
	}
	
	/**
	 * Test.
	 * 
	 * @param args
	 */
	public static void main(String[] args)
	{
		final List<Lebewesen> lebewesenList =
			new ArrayList<Lebewesen>();
		
		final Lebewesen adler =
			new Lebewesen(
					"Adler" ,
					// zur Vereinfachung kann der Adler nur 
					// fliegen, nicht laufen
					new FliegBewegung() ,
					new TierischErnaehrung() ,
					new VogelFortpflanzung() );
		
		lebewesenList.add( adler );
		
		final Lebewesen pinguin =
			new Lebewesen(
					"Pinguin" ,
					new MultiBewegung(
							new LaufBewegung() ,
							new SchwimmBewegung() ) ,
					// ich hoffe es stimmt, dass Pinguine
					// Tiere und Pflanzen fressen
					new MultiErnaehrung(
							new TierischErnaehrung() ,
							new PflanzlichErnaehrung() ) ,
					new VogelFortpflanzung() );
		
		lebewesenList.add( pinguin );
		
		final Lebewesen elefant =
			new Lebewesen(
					"Elefant" ,
					// zur Vereinfachung kann der Elefant nur 
					// laufen und nicht schwimmen
					new LaufBewegung() ,
					new PflanzlichErnaehrung() ,
					new SaeugetierFortpflanzung() );
		
		lebewesenList.add( elefant );
		
		final Lebewesen tiger =
			new Lebewesen(
					"Tiger" ,
					new MultiBewegung(
							new LaufBewegung() ,
							new SchwimmBewegung() ) ,
					new TierischErnaehrung() ,
					new SaeugetierFortpflanzung() );
		
		lebewesenList.add( tiger );
		
		final Lebewesen delphin =
			new Lebewesen(
					"Delphin" ,
					new SchwimmBewegung() ,
					new MultiErnaehrung(
							new TierischErnaehrung() ,
							new PflanzlichErnaehrung() ) ,
					new SaeugetierFortpflanzung() );
		
		lebewesenList.add( delphin );
		
		final Lebewesen hering =
			new Lebewesen(
					"Hering" ,
					new SchwimmBewegung() ,
					new MultiErnaehrung(
							new TierischErnaehrung() ,
							new PflanzlichErnaehrung() ) ,
					new FischFortpflanzung() );
		
		lebewesenList.add( hering );
		
		/* Und jetzt geht es los.
		 */
		for ( Lebewesen lebewesen : lebewesenList )
		{
			lebewesen.bewegDich();
			lebewesen.ernaehreDich();
			lebewesen.fortpflanzDich();
			
			System.out.println();
		}
	}

}
